//
//  BFRecordScreenCameraManager.swift
//  BFRecordScreenKit
//
//  Created by 胡志强 on 2022/1/13.
//

/*

 时长是根据摄像头文件时长相加得来的，预览/合成时需要考虑录音的时长问题

 videosticker 已经存储好了，在合成时，需要更新下进入的时间就可以了，不需要再切割了

 录音1s失败和摄像头1s失败不同步，需要分开计算

 */

import BFCommonKit
import BFMediaKit
import BFUIKit
import CoreMedia
import Foundation
import GPUImage
import SwiftUI
import UIKit

let vpath = recordVideosDirectory + "camera_writer.mov"
let videoSize = CGSize(width: 1080, height: 1920)

class BFRecordScreenCameraManager: BFRecordScreenBaseManager {
    //
    var rendView = GPUImageView()
    var cameraProgressV: BFCameraProgressView?

    override var recordItem: BFRecordItemModel? {
        didSet {
            cameraProgressV?.recordItem = recordItem
        }
    }

    // 录制完成回调
    var hasInitCallBack: (() -> Void)?
    var cameraFlipHandle: ((Bool) -> Void)?
    var recordProgressCallBack: ((CMTime) -> Void)?

    var startTime = Date()
    var videoModel = PQEditVisionTrackMaterialsModel()
    var playingVoiceMode : PQVoiceModel?
    // 用于打开摄像头初始流程
    static var initOpenCamera: Bool = true

    // timer循环每次录制增长的时长
    var increaseTime: CMTime = .zero

    fileprivate var timerr: Timer?

    var writerlock = false
    var isFrontCamera = false
    // 录制时音频结果
    var recordFinishedResult = false
    var hadDeleteVideoOfFailed = false

    var movieWrite: GPUImageMovieWriter?

    lazy var camera: GPUImageStillCamera? = {
        let camera = GPUImageStillCamera(sessionPreset: AVCaptureSession.Preset.hd1920x1080.rawValue, cameraPosition: AVCaptureDevice.Position.back)
        camera?.outputImageOrientation = UIInterfaceOrientation.portrait
        camera?.delegate = self
        return camera

    }()

    lazy var gropQueue: DispatchGroup = {
        let group = DispatchGroup() // 创建group
        return group
    }()

    var groupCount = 0
    
    // 录音文件是否正在播放
    var audioPlayerIsPlaying:Bool = false

    lazy var avplayer: AVPlayer = {
        let p = AVPlayer(playerItem: nil)
        
        return p
    }()

    lazy var playerCoverIV: UIImageView = {
        let iv = UIImageView()
        iv.frame = UIScreen.main.bounds
        iv.layer.addSublayer(playerLayer)
        iv.contentMode = .scaleAspectFit
        iv.backgroundColor = .black
        return iv
    }()

    lazy var playerLayer: AVPlayerLayer = {
        let layer = AVPlayerLayer(player: avplayer)
        layer.frame = UIScreen.main.bounds
        return layer
    }()

    // add by ak 延迟播放任务是否已经取消 , YES：（1,avplayer 播放结束时，2，暂停时），NO:(1,avplayer开始播放时)
    var dispatchWorkItemIsCancel: Bool = false

    // MARK: 重放

    override func resetEnv() {
        guard let camera = camera else {
            cShowHUB(superView: nil, msg: "option_fail_camera".BFLocale)
            return
        }
        resetThumbImg()

        initplayer()

        if let count = recordItem?.voiceStickers.count, count > 0 {
            currentAssetProgress = recordItem!.materialDuraion
            cameraProgressV?.isHidden = false
            progreddL?.isHidden = false
            progreddL?.text = CMTimeGetSeconds(currentAssetProgress).formatDurationToHMS()
            cameraProgressV?.progressView.scrollRectToVisible(CGRect(x: (cameraProgressV?.progressView.contentSize.width)! - 1, y: (cameraProgressV?.progressView.contentSize.height)! - 1, width: 1, height: 1), animated: false)
            dele?.isEndPlay = true
        } else {
            currentAssetProgress = .zero

            cameraProgressV?.isHidden = true
            progreddL?.isHidden = true
            playerCoverIV.isHidden = true
        }

        camera.startCapture()
        initerlizeWriter()

        camera.addTarget(filter)
        filter.addTarget(rendView)

        if BFRecordScreenCameraManager.initOpenCamera {
            startRecord()
            DispatchQueue.main.asyncAfter(deadline: .now() + 0.3, execute: { [weak self] in
                guard let wself = self else { return }
                while wself.groupCount > 0 {
                    wself.groupCount -= 1
                    wself.gropQueue.leave()
                }

                wself.endRecord()
            })
        }

        super.resetEnv()
    }

    func initplayer() {
        if let preView = playView {
            preView.addSubview(playerCoverIV)
        }
    }

    func cameraFlip() {
//        [self.camera rotateCamera];
        isFrontCamera = !isFrontCamera
        cameraFlipHandle?(isFrontCamera)
        camera?.horizontallyMirrorFrontFacingCamera = isFrontCamera
        camera?.rotateCamera()
    }

    override func startRecord() {
        gropQueue.enter() // 录音回调等待
        groupCount += 1

        guard let movieWrite = movieWrite, !writerlock else {
            cShowHUB(superView: nil, msg: "option_fail_record".BFLocale)
            return
        }

        playerCoverIV.isHidden = true
        recordFinishedResult = false

        super.startRecord()

        startTime = Date()
        hadDeleteVideoOfFailed = false

        videoModel = PQEditVisionTrackMaterialsModel()
        videoModel.thumImgs = [UIImage]()
        videoModel.timelineCMIn = currentAssetProgress
        videoModel.timelineCMOut = currentAssetProgress
        videoModel.locationPath = "nil"

        // 为了第一时间能更新collection view数据
        if !BFRecordScreenCameraManager.initOpenCamera {
            // 将model插入正确位置
            if let index = recordItem?.videoStickers.firstIndex(where: { mod in
                mod.timelineCMOut.seconds - currentAssetProgress.seconds > 0.7
            }) {
                recordItem?.videoStickers.insert(videoModel, at: index)
            } else {
                recordItem?.videoStickers.append(videoModel)
            }
            progreddL?.isHidden = false
            cameraProgressV?.isHidden = false
        }
        movieWrite.startRecording()
        timerr?.invalidate()
        timerr = Timer.scheduledTimer(withTimeInterval: 0.01, repeats: true, block: { [weak self] _ in
            guard let wself = self else { return }
            if BFRecordScreenCameraManager.initOpenCamera {
                return
            }

            // MARK: 进度计时器

            let currDur = CMTime(seconds: wself.videoModel.timelineCMIn.seconds + movieWrite.duration.seconds, preferredTimescale: 1000)
            if CMTimeCompare(currDur, wself.currentAssetProgress) > 0 {
                wself.increaseTime = currDur - wself.currentAssetProgress
                wself.currentAssetProgress = currDur
                wself.videoModel.timelineCMOut = currDur
                wself.recordProgressCallBack?(currDur)
                wself.updateUI(progress: currDur)
//                BFLog(1, message: "frame: \(wself.currentAssetProgress.seconds)")
            }

//            wself.getThumImage()
        })
        RunLoop.current.add(timerr!, forMode: .common)

        gropQueue.enter() // 录像
        groupCount += 1

        // MARK: 结束录制回调

        gropQueue.notify(queue: DispatchQueue.main) { [weak self] in
            guard let wself = self else { return }
            if BFRecordScreenCameraManager.initOpenCamera {
                BFRecordScreenCameraManager.initOpenCamera = false
                return
            }
            let cameraSuccess = wself.recordFinishedResult && ((wself.videoModel.timelineCMOut - wself.videoModel.timelineCMIn).seconds > 1)
            if cameraSuccess {
                wself.increaseTime = wself.videoModel.timelineCMOut - wself.currentAssetProgress

                wself.currentAssetProgress = wself.videoModel.timelineCMOut

                wself.recordEndCallBack?(true, wself.videoModel)
                // 更新各录音时间点，在插入的时候
                wself.coculationTotalDur()

                wself.updateUI(progress: wself.currentAssetProgress)
                if let img = wself.videoModel.thumImgs?.last {
                    wself.cameraProgressV?.changeSepline(img: img)
                }
            } else {
                wself.recordEndCallBack?(false, nil)
                wself.videoModel.locationPath = "nil"
                wself.cameraProgressV?.newRecordLine = true
            }
        }
    }

    override func endRecord() {
        super.endRecord()

        writerlock = true
        timerr?.invalidate()
        timerr = nil
//        BFLog(1, message: " 拍摄时长 \(Date().timeIntervalSince(startTime))")
        let su = BFRecordScreenCameraManager.initOpenCamera
        movieWrite?.finishRecording(completionHandler: { [weak self] in
            guard let wself = self else { return }

            if FileManager.default.fileExists(atPath: vpath) {
                if !su {
                    if let finalPath = PQBridgeObject.p_setupFileRename(vpath), finalPath.count > 0 {
                        let dur = AVURLAsset(url: URL(fileURLWithPath: finalPath))

                        wself.videoModel.timelineCMOut = wself.videoModel.timelineCMIn + CMTime(seconds: dur.duration.seconds, preferredTimescale: 1000)
                        BFLog(1, message: "文件时长camera:\(wself.recordItem?.videoStickers.count ?? 0),   \(wself.videoModel.timelineCMIn.seconds)~\(wself.videoModel.timelineCMOut.seconds),  \(dur.duration.seconds)")
                        wself.videoModel.locationPath = finalPath
                    } else {
                        cShowHUB(superView: nil, msg: "shoot_tips_least".BFLocale)
                    }
                    wself.gropQueue.leave()
                }
            }

            // 重置writer，提前为下次录制做准备
            DispatchQueue.global().async {[weak self] in
                guard let wself = self else { return }
                
                wself.initerlizeWriter()
            }
            wself.writerlock = false
        })
    }

    override func recordFinished(isSuccess: Bool) {
        recordFinishedResult = isSuccess
        groupCount = groupCount - 1
        gropQueue.leave()
    }

    override func deleteRecord(at: CMTime) {
        if let videoS = recordItem?.videoStickers.enumerated().first(where: { mod in
            CMTimeCompare(mod.1.timelineCMIn, at) <= 0 && CMTimeCompare(mod.1.timelineCMOut, at) > 0
        }), let item = recordItem {
            item.thumbImgs.removeAll(where: { img in
                videoS.1.thumImgs?.contains(img) ?? false
            })
            item.videoStickers.remove(at: videoS.0)

            if var event = dele?.events.last {
                event.deletedCameras = [videoS.1]
                dele?.events.removeLast()
                dele?.events.append(event)
            }

            let currDuration = videoS.1.timelineCMOut - videoS.1.timelineCMIn
            currentAssetProgress = videoS.1.timelineCMIn
            // 更新音频的时间点
            for model in item.voiceStickers {
                if CMTimeCompare(currentAssetProgress, model.startCMTime) <= 0 {
                    model.startCMTime = model.startCMTime - currDuration
                    model.endCMTime = model.endCMTime - currDuration
                    BFLog(1, message: "camera删除后更新voice time：\(model.startCMTime.seconds) ~ \(model.endCMTime.seconds)")
                    item.titleStickers.forEach { tm in
                        if tm.recordId == model.recordId {
                            tm.timelineIn = tm.timelineIn - currDuration
                            tm.timelineOut = tm.timelineOut - currDuration
                        }
                    }
                    
                    let vcList = recordItem!.voiceChangeStickers.filter { vc in
                        vc.recordId == model.recordId
                    }
                    for voice in vcList {
                        voice.startCMTime = voice.startCMTime - currDuration
                        voice.endCMTime = voice.endCMTime - currDuration
                    }
                }
            }
            // 更新录像的时间点
            for model in item.videoStickers {
                if CMTimeCompare(currentAssetProgress, model.timelineCMIn) < 0 {
                    model.timelineCMIn = model.timelineCMIn - currDuration
                    model.timelineCMOut = model.timelineCMOut - currDuration
                    BFLog(1, message: "camera删除后更新video time：\(model.timelineCMIn.seconds) ~ \(model.timelineCMOut.seconds)")
                }
            }

            item.materialDuraion = item.videoStickers.last?.timelineCMOut ?? .zero
        }
        changeRecordMaterail()
        BFLog(1, message: "delete: video- \(recordItem?.videoStickers.count ?? 0), voice-\(recordItem?.voiceStickers.count ?? 0)")
    }

    func coculationTotalDur() {
        if videoModel.locationPath != "nil" {
            recordItem?.videoStickers.forEach({ mod in
                if mod != videoModel, mod.timelineCMOut.seconds > videoModel.timelineCMIn.seconds {
                    mod.timelineCMIn = mod.timelineCMIn + videoModel.timelineCMOut - videoModel.timelineCMIn
                    mod.timelineCMOut = mod.timelineCMOut + videoModel.timelineCMOut - videoModel.timelineCMIn
                }
            })
        }

//        resetThumbImg()

        recordItem?.materialDuraion = recordItem?.videoStickers.reduce(.zero, { partialResult, voice in
            voice.timelineCMOut - voice.timelineCMIn + partialResult
        }) ?? .zero
    }

    // 录制失败 ， 不足一秒时恢复上一次状态
    func revertLast(_ modIndex: Int = -1) {
        hadDeleteVideoOfFailed = true

        var vmod: PQEditVisionTrackMaterialsModel?
        if modIndex >= 0 && modIndex < recordItem?.videoStickers.count ?? 0 {
            vmod = recordItem?.videoStickers[modIndex]
        } else {
//            vmod = recordItem?.videoStickers.first(where: { mod in
//                CMTimeCompare(mod.timelineCMIn, currentAssetProgress) >= 0
//            })
            vmod = videoModel
        }
        if vmod != nil {
            vmod!.timelineCMOut = vmod!.timelineCMIn
            currentAssetProgress = vmod!.timelineCMOut
//            vmod.thumImgs?.removeAll()
            recordItem?.videoStickers.removeAll(where: { mod in
                mod == vmod
            })
            BFLog(1, message: "count videoStickers: \(recordItem?.videoStickers.count ?? 0)")
        }

        resetThumbImg()
    }

    func resetThumbImg() {
//        videoModel.thumImgs?.removeAll()
        let vidos = recordItem?.videoStickers
        var tmpThumbs = [UIImage]()
        vidos?.forEach({ mod in
            if let imgs = mod.thumImgs {
                tmpThumbs.append(contentsOf: imgs)
            }
        })
        recordItem?.thumbImgs = tmpThumbs
        DispatchQueue.main.async { [weak self] in
            guard let wself = self else { return }
            wself.cameraProgressV?.resetThumIV()
        }
    }

    // 使用avplayer进行播放视频和音频
    override func play() {
        guard playView != nil else {
            return
        }

        playerCoverIV.isHidden = false

        camera?.pauseCapture()

        // 100ms以内的重头开始播放
        if (recordItem?.materialDuraion.seconds)! - currentAssetProgress.seconds < 0.1 {
            currentAssetProgress = .zero
            locationTo(time: currentAssetProgress)
            avplayer.replaceCurrentItem(with: nil)
        }
        
        startTimer(isPlay: true)
    }

    func playRecordVoice(needPlay: Bool = true) {

        // 三种状态值ff
//        case paused = 0
//        case waitingToPlayAtSpecifiedRate = 1
//        case playing = 2
        if (BFAudioPlayer.shared.player?.isPlaying ?? false || audioPlayerIsPlaying){
//            BFLog(2, message: "is playering 本次查找无效，有播放任务")
            return
        }
//        BFLog(2,message: "开始查找！！！！当前进度:\(currentAssetProgress.seconds)")

        // 播放音频
        var useVoiceStickers = [PQVoiceModel]()
        if (recordItem?.voiceChangeStickers.count ?? 0) > 0 {
            useVoiceStickers = recordItem?.voiceChangeStickers ?? [PQVoiceModel]()
        } else {
            useVoiceStickers = recordItem?.voiceStickers ?? [PQVoiceModel]()
        }

        if let mod = useVoiceStickers.first(where: { m in
            CMTimeCompare(m.startCMTime, currentAssetProgress) <= 0 && CMTimeCompare(m.endCMTime, currentAssetProgress) > 0

        }) {
            BFLog(message: "找到音频开始时间:\(mod.startCMTime.seconds)结束 \(mod.endCMTime.seconds) currentAssetProgress is \(currentAssetProgress.seconds) wavFilePath:\(mod.wavFilePath ?? "")")

            // 当前播放的文件
            if playingVoiceMode == mod {
                BFLog(2,message: "找查到mod的已经播放完成同一个 wavFilePath:\(mod.wavFilePath ?? "")")
                return
            }
            playingVoiceMode = mod
            
            let cureentPalyURlString = BFAudioPlayer.shared.currentAudioPathURL?.absoluteString ?? ""
            if cureentPalyURlString.replacingOccurrences(of: "file://", with: "") == mod.wavFilePath && audioPlayerIsPlaying{
                BFLog(2, message: "命中的和播放的文件路径一样？ wavFilePath:\(mod.wavFilePath ?? "")")
//                audioPlayerIsPlaying = true
//                BFAudioPlayer.shared.playOrPause()
                
//                if !(audioPlayerIsPlaying) {
////                        audioPlayerIsPlaying = true
                    BFAudioPlayer.shared.setPlayingTime((currentAssetProgress - mod.startCMTime).seconds)
////                        BFAudioPlayer.shared.playOrPause()
//                }


                return
            }
            
           let ttsAudioAsset = AVURLAsset(url: URL(fileURLWithPath: mod.wavFilePath), options: avAssertOptions)
            
           let outTime =  ttsAudioAsset.duration.seconds - (mod.endCMTime - mod.startCMTime).seconds
            BFAudioPlayer.shared.delegate = self
            BFAudioPlayer.shared.configPlayer(audioPathURL: URL(fileURLWithPath: mod.wavFilePath))
            //如果音频比原来的字幕时间要长，动态加速保证和字幕时长一致。 0.1多算一点音频时长，也就是让更快速度播放一些 XXXX
            BFAudioPlayer.shared.player?.rate = Float(outTime > 0 ? (ttsAudioAsset.duration.seconds + 0.1) / (mod.endCMTime - mod.startCMTime).seconds : 1.0)
            
            let tt = currentAssetProgress - mod.startCMTime
            BFLog(1, message: "record - tt: \(tt.seconds), curr:\(currentAssetProgress.seconds) \(needPlay) rate:\(BFAudioPlayer.shared.player?.rate  ?? -1) 音频比字幕显示时间长 :\(outTime) \(mod.wavFilePath ?? "")")
            if CMTimeCompare(tt, .zero) == 0 {
                if needPlay {
                    audioPlayerIsPlaying = true
                    BFAudioPlayer.shared.playOrPause()
                }
                return
            }
            audioPlayerIsPlaying = true
            BFAudioPlayer.shared.setPlayingTime(tt.seconds)
            BFAudioPlayer.shared.playOrPause()
 
        } else {
            BFLog(message: "没有找到即将播放的录制音频文件？\(currentAssetProgress.seconds)")
        }
    }

    func prepareToPlayNext(needPlay: Bool = true) {
        
        if let mod = recordItem?.videoStickers.first(where: { mode in
            CMTimeCompare(mode.timelineCMIn, currentAssetProgress) <= 0 && CMTimeCompare(currentAssetProgress, mode.timelineCMOut) < 0
        }) {
            // true 代表需要换源
            let su = (avplayer.currentItem?.asset as? AVURLAsset)?.url.lastPathComponent != (mod.locationPath as NSString?)?.lastPathComponent
            
            if !su {
                if avplayer.timeControlStatus == .paused { // 同源且暂停的时候，需要修改进度
                    setCoverImage(currentAssetProgress)
                    
                    let tt = currentAssetProgress - mod.timelineCMIn
                    avplayer.currentItem?.seek(to: tt, toleranceBefore: CMTime(value: 1, timescale: 1_000_000), toleranceAfter: CMTime(value: 1, timescale: 1_000_000), completionHandler: { [weak avplayer] _ in

                        if needPlay {
                            avplayer?.play()
                        }
                    })
                    

                }
                return
            }
            
            setCoverImage(currentAssetProgress)

//            locationTo(time: currentAssetProgress)
            avplayerReplaceItem(newItem: AVPlayerItem(url: URL(fileURLWithPath: mod.locationPath)))
            BFLog(1, message: "当前：\(currentAssetProgress.seconds), mod:\(mod.timelineCMIn.seconds) ~ \(mod.timelineCMOut.seconds)")
            let tt = currentAssetProgress - mod.timelineCMIn
            BFLog(1, message: "asset - tt: \(tt.seconds), curr:\(currentAssetProgress.seconds)")
            
            if CMTimeCompare(tt, (avplayer.currentItem?.duration ?? .zero)) == 0 {
                if needPlay {
                    avplayer.play()
                }
            } else {
                avplayer.currentItem?.seek(to: tt, toleranceBefore: CMTime(value: 1, timescale: 1_000_000), toleranceAfter: CMTime(value: 1, timescale: 1_000_000), completionHandler: { [weak avplayer] _ in

                    if needPlay {
                        avplayer?.play()
                    }
                })
            }
        } else {
            avplayerReplaceItem(newItem: nil)

            locationTo(time: currentAssetProgress)
            playerCoverIV.isHidden = true
            BFLog(1, message: "prepareToPlayNext: 没找到 \(currentAssetProgress.seconds)")
        }
    }

    func avplayerReplaceItem(newItem: AVPlayerItem?) {
        if let item = avplayer.currentItem {
            if (item.asset as? AVURLAsset)?.url == (newItem?.asset as? AVURLAsset)?.url {
                return
            }
            NotificationCenter.default.removeObserver(self as Any, name: .AVPlayerItemDidPlayToEndTime, object: item)
        }

        avplayer.replaceCurrentItem(with: newItem)
        if newItem == nil {
            return
        }

        NotificationCenter.default.addObserver(forName: .AVPlayerItemDidPlayToEndTime, object: avplayer.currentItem, queue: .main) { [weak self] _ in
            guard let wself = self else {
                return
            }

            BFLog(1, message: "endPlay:\(wself.currentAssetProgress.seconds), 总时长:\(wself.recordItem!.materialDuraion.seconds), currplayer:\(wself.avplayer.currentItem?.currentTime().seconds ?? 0)")

            // 判断录音是否还在播放，如果在播放停止
            if BFAudioPlayer.shared.player?.isPlaying ?? false {
                wself.recordPlayer?.pause()
            }
//            // 如果离结束大于0.7s，代表还有下一段视频
//            if ((wself.recordItem?.materialDuraion ?? .zero) - wself.currentAssetProgress).seconds > 0.7 {
////                wself.findNextModel()
////                wself.setCoverImage(wself.currentAssetProgress)
//
////                wself.prepareToPlayNext()
//            } else {
//                // 播放到末尾了
//                wself.dele?.isEndPlay = true
//                wself.dele?.pause()
////                wself.playerLayer.removeFromSuperlayer()
//                wself.playerCoverIV.isHidden = true
//            }
        }
    }

    override func pause() {
        dispatchWorkItemIsCancel = true
        avplayer.pause()
       
        BFAudioPlayer.shared.pause()

        audioPlayerIsPlaying = false

        playingVoiceMode = nil

        timerr?.invalidate()
        timerr = nil

        if currentAssetProgress == recordItem?.materialDuraion {
            camera?.resumeCameraCapture()
            playerCoverIV.isHidden = true
        }
    }

    override func dragEnd() {
        // 播放视频
        if let mod = recordItem?.videoStickers.first(where: { m in
            CMTimeCompare(m.timelineCMIn, currentAssetProgress) <= 0 && CMTimeCompare(m.timelineCMOut, currentAssetProgress) > 0
        }), (CMTimeCompare(currentAssetProgress, (recordItem?.materialDuraion ?? .zero)) < 0) {
            BFLog(1, message: "drag end:\(currentAssetProgress.seconds), mod:\(mod.timelineCMIn)")

            playerCoverIV.image = getThumbImageAtTime(urlAsset: AVURLAsset(url: URL(fileURLWithPath: mod.locationPath)), time: currentAssetProgress - mod.timelineCMIn)
            playerCoverIV.isHidden = false
            
//            avplayerReplaceItem(newItem: AVPlayerItem(url: URL(fileURLWithPath: mod.locationPath)))

            prepareToPlayNext(needPlay: false)
        } else {
            playerCoverIV.isHidden = true
            avplayerReplaceItem(newItem: nil)
        }
    }

    // 获取下一个视频的封面图
    func setCoverImage(_ dur: CMTime) {
        if let mod = recordItem?.videoStickers.first(where: { m in
            CMTimeCompare(m.timelineCMIn, (dur + CMTime(seconds: 0.5, preferredTimescale: 1000))) <= 0 && CMTimeCompare(m.timelineCMOut, (dur + CMTime(seconds: 0.5, preferredTimescale: 1000))) > 0
        }) {
            playerCoverIV.image = PQVideoSnapshotUtil.videoSnapshot(videoURL: URL(fileURLWithPath: mod.locationPath), time: .zero)
        }
    }

    //
    override func changeRecordMaterail() {
        //
        if recordItem?.videoStickers.count ?? 0 == 0 {
            currentAssetProgress = .zero
            resetEnv()
        } else {
            cameraProgressV?.resetThumIV()
            cameraProgressV?.progressView.contentOffset = CGPoint(x: currentAssetProgress.seconds * 70.0 / 5.0, y: 0)
            progreddL?.text = CMTimeGetSeconds(currentAssetProgress).formatDurationToHMS()

            cameraProgressV?.isHidden = false

//            prepareToPlayNext(needPlay: false)
//            avplayerReplaceItem(newItem: nil)
        }
    }

    override func clean() {
        filter.removeTarget(movieWrite)
        filter.removeTarget(rendView)
        camera?.removeTarget(filter)

        camera?.stopCapture()

        super.clean()
    }

    // MARK: - 私有方法

    /// 为播放/录制启动定时器
    /// - Parameter isPlay: 是播放么？ true为播放，false为录制
    func startTimer(isPlay: Bool) {
        let startPlayTime = Date()
        let startProgress = currentAssetProgress

        timerr?.invalidate()
        timerr = Timer.scheduledTimer(withTimeInterval: 0.01, repeats: true, block: { [weak self] _ in
            guard let wself = self else { return }

            let currTime = startProgress + CMTime(seconds: Date().timeIntervalSince(startPlayTime), preferredTimescale: 1000)

            if isPlay { // 点击播放
                wself.timerForPlay(currTime: currTime)
            } else { // 点击拍摄
                wself.timerForRecord(currTime: currTime)
            }

//            wself.getThumImage()
        })
        RunLoop.current.add(timerr!, forMode: .common)
    }

    func timerForPlay(currTime: CMTime) {
        currentAssetProgress = max(.zero, min(currTime, (recordItem?.materialDuraion ?? .zero)))
        dele?.progreddL.text = String(format: "%@", currentAssetProgress.seconds.formatDurationToHMS())
        dele?.progressThumV.progress = currentAssetProgress
        dele?.updateSubtitle(time: currentAssetProgress)
        
        let p = CGPoint(x: currentAssetProgress.seconds * 70.0 / 5.0, y: 0)
        cameraProgressV?.progressView.contentOffset = p
        
//        BFLog(1, message: "curr: \(currentAssetProgress.seconds)")
        
        if currTime.seconds >= (recordItem?.materialDuraion.seconds ?? 0) {
            dele?.isEndPlay = true
            isPlaying = false
            dele?.pause()
            playerCoverIV.isHidden = true
            return
        }

        // 检测录音的
//        checkCurrentPlayStatus()
        // 检测视频的
        prepareToPlayNext()
        // 新的检测录音的
        playRecordVoice()
    }

    func timerForRecord(currTime _: CMTime) {
        if BFRecordScreenCameraManager.initOpenCamera {
            return
        }

        // MARK: 进度计时器

        let currDur = CMTime(seconds: videoModel.timelineCMIn.seconds + (movieWrite?.duration.seconds ?? 0), preferredTimescale: 1000)
        if CMTimeCompare(currDur, currentAssetProgress) > 0 {
            increaseTime = currDur - currentAssetProgress
            currentAssetProgress = currDur
            videoModel.timelineCMOut = currDur
            recordProgressCallBack?(currDur)
            updateUI(progress: currDur)
//                BFLog(1, message: "frame: \(currentAssetProgress.seconds)")
        }
    }

    // 定位到某个时间点，相应进度条的UI更新
    func locationTo(time: CMTime) {
        var toTime = CMTime(seconds: time.seconds, preferredTimescale: 1000)
        if CMTimeCompare(time, recordItem!.materialDuraion) > 0 {
            toTime = recordItem!.materialDuraion
            currentAssetProgress = toTime
        }
        progreddL?.text = CMTimeGetSeconds(currentAssetProgress).formatDurationToHMS()

        let p = CGPoint(x: currentAssetProgress.seconds * 70.0 / 5.0, y: 0)
        cameraProgressV?.progressView.contentOffset = p
    }

    /// 录制时更新录音进度
    /// - Parameter progress: 当前的进度
    func updateUI(progress: CMTime) {
        DispatchQueue.main.async { [weak self] in
            guard let wself = self else { return }

            wself.progreddL?.text = CMTimeGetSeconds(progress).formatDurationToHMS()

            // 刷新之后的sticker
            if wself.recordItem?.thumbImgs.count ?? 0 > 0, let sticker = wself.recordItem?.videoStickers.first(where: { mod in
                CMTimeCompare(mod.timelineCMIn, progress) <= 0 && CMTimeCompare(mod.timelineCMOut, progress) >= 0
            }) {
                wself.cameraProgressV?.updateCellWidth(sticker: sticker, progress: progress)
            }
        }
    }

    // 添加新的缩略图
    func addNewThumb(_ img: UIImage) {
        if let progressThumV = cameraProgressV {
            progressThumV.addThumbImages(img)
        }
    }

    // 每次录制完一段视频，都要重新初始化 movieWrite
    func initerlizeWriter() {
        if FileManager.default.fileExists(atPath: vpath) {
            try? FileManager.default.removeItem(atPath: vpath)
        }
        if movieWrite != nil {
            movieWrite?.endProcessing()
            filter.removeTarget(movieWrite)
            movieWrite = nil
        }

        movieWrite = GPUImageMovieWriter(movieURL: URL(fileURLWithPath: vpath), size: videoSize)
        filter.addTarget(movieWrite)
    }
}

extension BFRecordScreenCameraManager: BFAudioPlayerDelegate{
    
    // 播放完成
    func audioPlayerDidFinish(){
        BFLog(message: "audioPlayerDidFinish")
        audioPlayerIsPlaying = false
        BFAudioPlayer.shared.pause()
    }
    
    // 播放开始
    func audioPlayerStart(){
        
    }
    // 播放进度
    func playProgress(currentTime: TimeInterval, progress: Double){
        
//        BFLog(message: "currentTime is \(currentTime) progress\(progress) \(BFAudioPlayer.shared.currentAudioPathURL?.absoluteString ?? "")")
    }
    
}

extension BFRecordScreenCameraManager: GPUImageVideoCameraDelegate {
    func willOutputSampleBuffer(_ sampleBuffer: CMSampleBuffer!) {
        if !recording || BFRecordScreenCameraManager.initOpenCamera {
            return
        }
        let dur = CMTime(seconds: movieWrite?.duration.seconds ?? 0, preferredTimescale: 1000)
        // 获取缩略图
        let coculationCount = Int(dur.seconds / 5.0)
        if coculationCount >= (videoModel.thumImgs?.count ?? 0) {
            imageFromBuffer(sampleBuffer) { [weak self] img in
                guard let wself = self else { return }

                if img != nil {
//                    BFLog(1, message: "取一张缩略图出来 \(dur.seconds)， \(wself.recordItem?.thumbImgs.count ?? 0)")
                    wself.videoModel.thumImgs?.append(img!)
                    if let videoStickers = wself.recordItem?.videoStickers {
                        wself.recordItem?.thumbImgs = videoStickers.map({ mod in
                            mod.thumImgs!
                        }).flatMap { $0 }
                    }
                    wself.addNewThumb(img!)
                }
            }
        }
    }

    func imageFromBuffer(_ buff: CMSampleBuffer, callback: ((UIImage?) -> Void)) {
        if let buffer = CMSampleBufferGetImageBuffer(buff) {
            let ciImage = CIImage(cvPixelBuffer: buffer)
            let img = UIImage(ciImage: ciImage.oriented(forExifOrientation: 6),
                              scale: cScreenWidth / 70.0,
                              orientation: isFrontCamera ? .upMirrored : .up)
            if let data = img.jpegData(compressionQuality: 0.3) {
                callback(UIImage(data: data))
            }
        } else {
            callback(nil)
        }
    }
}
