//
//  BFIndirectionProgressView.swift
//  BFRecordScreenKit
//
//  Created by SanW on 2021/12/18.
//  Copyright © 2021 BytesFlow. All rights reserved.
//

import BFCommonKit
import BFMediaKit
import UIKit

class BFIndirectionProgressView: UIView {
    var indirecColor: UIColor = UIColor.clear
    var themeColor: UIColor = UIColor.hexColor(hexadecimal: "#389AFF")
    var progressHeight: CGFloat = 6
    var tmpPercentW: CGFloat = 0
    var deletedView: UIView?
    var percenWidth: CGFloat {
        get {
            if tmpPercentW.isNaN || tmpPercentW.isInfinite || (tmpPercentW <= 0 && totalDuration > 0) {
                return frame.width / totalDuration
            } else {
                return tmpPercentW
            }
        }
        set {
            tmpPercentW = newValue
        }
    }

    var totalDuration: Float64 = 0
    var currentItem: UIView? // 当前的Item

    override private init(frame: CGRect) {
        super.init(frame: frame)
    }

    init(frame: CGRect, indirecColor: UIColor = UIColor.clear, themeColor: UIColor = UIColor.hexColor(hexadecimal: "#389AFF"), percenWidth: CGFloat = 0, progressHeight: CGFloat = 6, totalDuration: Float64 = 0) {
        super.init(frame: frame)
        self.indirecColor = indirecColor
        self.themeColor = themeColor
        self.totalDuration = totalDuration
        self.progressHeight = progressHeight
        self.percenWidth = percenWidth
    }

    required init?(coder _: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }

    /// 重绘view
    /// - Parameter items: <#items description#>
    ///  - Parameter  percenWidth: 图片为固定值要传入，视频的话可以计算，也可以传入
    func resetAllSubViews(items: [PQVoiceModel]?, percenWidth: CGFloat = 0, totalDuration: Float64) {
        frame.size.width = (superview as? UIScrollView)?.contentSize.width ?? 0
        self.totalDuration = totalDuration
        self.percenWidth = percenWidth
        deletedView = nil
        currentItem = nil
        subviews.forEach { vv in
            vv.removeFromSuperview()
        }
        
        var lastM : PQVoiceModel?
        var lastv : BFThumImageView?
        items?.forEach { [weak self] model in
            
            let v = createItemView(minX: model.startCMTime.seconds * CGFloat(self?.percenWidth ?? 0), width: (model.endCMTime.seconds - model.startCMTime.seconds) * CGFloat(self?.percenWidth ?? 0))
//            BFLog(1, message: "equ:\((lastM?.endCMTime ?? .zero).seconds), \(model.startCMTime.seconds), \(CMTimeCompare(lastM?.endCMTime ?? .zero, model.startCMTime) < 0 ? "==" : "!=")")
            if let lastM = lastM, CMTimeCompare(lastM.endCMTime, model.startCMTime) < 0 {
                lastv?.isHiddenBord = true
            }
            lastM = model
            lastv = v
        }
        (subviews.last as? BFThumImageView)?.isHiddenBord = true
    }

    /// 设置进度
    /// - Parameters:
    ///   - _:
    ///   - start: 开始录制的时间
    ///   - progress: 当前录制时长
    func setProgress(start: CMTime = .zero, progress: Float64, isInsert:Bool = false) {
        if start.seconds * percenWidth >= frame.width {
            frame.size.width = (superview as? UIScrollView)?.contentSize.width ?? 0
        }

        detectionAndCreateItem(start: start, progress: progress)

        if isInsert {
            if let currentItem = currentItem {
                let oldwidth = currentItem.frame.width
                currentItem.frame.size.width = progress < 0 ? 0 : progress * percenWidth
                BFLog(message: "当前view：\(String(describing: currentItem))")
                
                // 插入后后边的视图后移
                subviews.forEach { subv in
                    if currentItem.frame.maxX < subv.frame.maxX {
                        subv.frame = subv.frame.offsetBy(dx: (currentItem.width) - oldwidth, dy: 0)
                    }
                }
            }
        }else {
            // 视频录制
            currentItem?.frame.size.width = progress < 0 ? 0 : progress * percenWidth
            BFLog(message: "当前view：\(String(describing: currentItem))")
            let vc = subviews.first { [weak self] sub in
                (self?.currentItem?.frame.minX ?? 0) < sub.frame.minX && sub.frame.minX < (self?.currentItem?.frame.maxX ?? 0)
            }
            if vc != nil {
                deletedView = vc
                vc?.removeFromSuperview()
                BFLog(message: "添加覆盖view-添加：deletedView = \(String(describing: deletedView))")
            }
        }
    }

    /// 检测并创建item
    /// - Parameter start: <#start description#>
    func detectionAndCreateItem(start: CMTime = .zero, progress: Float64) {
        BFLog(message: "检测并创建item：\(start)")
        if currentItem == nil {
            deletedView = nil

            currentItem = createItemView(minX: start.seconds * percenWidth)
            if subviews.count > 1 {
                let vvs = subviews.sorted { v1, v2 in
                    v1.rightX < v2.rightX
                }
                if let lastv = vvs.last(where: { vv in
                    (vv != currentItem) && abs(vv.rightX - currentItem!.x) < 0.01
                }) as? BFThumImageView {
                    lastv.isHiddenBord = false
                }
            }
        }
    }

    /// 检测当前view
    /// - Parameters:
    ///   - start: <#start description#>
    ///   - progress: <#progress description#>
    /// - Returns: <#description#>
    func detectionItem(start: CMTime = .zero, progress: Float64) -> BFThumImageView? {
        let newRange = CMTimeRange(start: start, duration: CMTime(seconds: progress, preferredTimescale: 1000))
        var currentIndex: Int?
        for (index, item) in subviews.enumerated() {
            let originRange = CMTimeRange(start: CMTime(seconds: item.frame.minX / percenWidth, preferredTimescale: 1000), end: CMTime(seconds: item.frame.width / percenWidth, preferredTimescale: 1000))
            if CMTimeRangeGetIntersection(originRange, otherRange: newRange).duration.seconds > 0 {
                currentIndex = index
                break
            }
        }
        if currentIndex != nil {
            BFLog(message: "检测存在取出")
            return subviews[currentIndex!] as? BFThumImageView
        } else {
            BFLog(message: "检测不存在重新创建：\(start)")
            let v = createItemView(minX: start.seconds * percenWidth)
            if subviews.count > 1 {
                let vvs = subviews.sorted { v1, v2 in
                    v1.rightX < v2.rightX
                }
                if let lastv = vvs.last(where: { vv in
                    (vv.x < v.x) && (vv.rightX == v.x)
                }) as? BFThumImageView {
                    lastv.isHiddenBord = false
                }
            }
            return v
        }
    }

    /// 录制结束后重制当前item
    func resetCurrentItem(start: CGFloat, end: CGFloat) {
        currentItem?.frame.origin.x = start * percenWidth
        currentItem?.frame.size.width = (end - start) * percenWidth

        if subviews.max(by: {$0.rightX < $1.rightX}) == currentItem {
            (currentItem as? BFThumImageView)?.isHiddenBord = true
        }
        currentItem = nil
        BFLog(message: "重置currentItem：\(start)")
    }

    /// 删除某个view
    /// - Parameter index: <#index description#>
    func deleteItem(index: Int = 0, isCurrent: Bool = false, isImage _: Bool = false) {
        BFLog(message: "添加覆盖view-不足一秒：\(isCurrent),deletedView = \(String(describing: deletedView))")
        if isCurrent {
            currentItem?.removeFromSuperview()
            currentItem = nil
            if deletedView != nil {
                BFLog(message: "添加view")
                addSubview(deletedView!)
            }
        } else {
            if index >= 0, index < subviews.count {
                subviews[index].removeFromSuperview()
            }
        }
        deletedView = nil
    }

    /// 创建一个view
    /// - Parameters:
    ///   - minX: <#minX description#>
    ///   - width: <#width description#>
    ///   - indirec: <#indirec description#>
    /// - Returns: <#description#>
    func createItemView(minX: CGFloat, width: CGFloat = 0, indirec: Bool = false) -> BFThumImageView {
        var newW = width
        if width.isNaN || width.isInfinite {
            newW = 0
        }
        let lineV = BFThumImageView(frame: CGRect(x: minX < 0 ? 0 : minX, y: 0, width: newW, height: progressHeight))
        lineV.backgroundColor = indirec ? indirecColor : themeColor
        addSubview(lineV)
        return lineV
    }
}
