//
//  File.swift
//  BFRecordScreenKit
//
//  Created by ak on 2021/12/2.
//  功能：录制头像 ，生成多个 MP4 文件。

import BFCommonKit
import BFMediaKit
import BFUIKit
import Foundation
import GPUImage

// 录制完成回调
typealias recordEndCallBack = (_ isSucess: Bool, _ material: PQEditVisionTrackMaterialsModel?) -> Void

class BFRecordAvatarView: UIView {
    // 画布
    var renderView: RenderView!
    // 摄像机
    var camera: Camera!
    // 是否是录制状态
    var isRecording = false
    // 视频输出源
    var movieOutput: MovieOutput?
    // 录制完成回调
    var recordEndCallBack: recordEndCallBack?
    // 录制完成生成的视频文件地址
    var recorderFilePath: String?
    // 开始录制时间
    var startRecordeTime: CMTime = .zero

    // 打开摄像头
    lazy var closedBtn: UIButton = {
        let btn = UIButton(type: .custom)
        btn.setImage(imageInRecordScreenKit(by: "CameraClosed"), for: .normal)
        btn.addTarget(self, action: #selector(cameraClosed), for: .touchUpInside)
        return btn
    }()

    // 预览时纹理大小&合成视频大小
    var videoPixelsSize: Size = Size(width: 120 * Float(UIScreen.main.scale), height: 120 * Float(UIScreen.main.scale))

    init(frame: CGRect, sessionPreset:AVCaptureSession.Preset = .high, location:PhysicalCameraLocation = .frontFacing) {
        super.init(frame: frame)
//        backgroundColor = .yellow

        renderView = RenderView(frame: bounds)
        addSubview(renderView)
        renderView.snp.makeConstraints { make in
            make.edges.equalToSuperview()
        }

        addSubview(closedBtn)
//        closedBtn.frame = CGRect(x: frame.maxX - 68, y: frame.maxY - 68, width: 68, height: 68)
        do {
            camera = try Camera(sessionPreset: sessionPreset, location: location, captureAsYUV: true)
//            camera.runBenchmark = true

//            let conertFilter = PQCornerFilter()
//            let cropFilter = Crop()
//            cropFilter.cropSizeInPixels = videoPixelsSize
//            cropFilter.cropSizeInPixels = Size(width: 1080, height: 1080)
//            cropFilter.locationOfCropInPixels = Position(0, (1920 - 1080) / 2)
//            camera  --> cropFilter --> conertFilter --> renderView
            recorderFilePath = recordVideosDirectory
            if !directoryIsExists(dicPath: recorderFilePath ?? "") {
                BFLog(message: "文件夹不存在 \(recorderFilePath ?? "")")
                createDirectory(path: recorderFilePath ?? "")
            }
            recorderFilePath!.append("recordAvatar_\(Date().timeIntervalSince1970).mp4")
            BFLog(message: "开始录制视频到： \(recorderFilePath ?? "")")

            movieOutput = try MovieOutput(URL: URL(fileURLWithPath: recorderFilePath ?? ""), size: videoPixelsSize, liveVideo: true)

            let cropFilter = Crop()
            let conertFilter = PQCornerFilter()

            cropFilter.cropSizeInPixels = videoPixelsSize
            cropFilter.locationOfCropInPixels = Position(0, 0)

            camera --> cropFilter --> conertFilter --> movieOutput!

//            camera.audioEncodingTarget = movieOutput
        } catch {
            fatalError("Could not initialize rendering pipeline: \(error)")
        }

        let Drag = UIPanGestureRecognizer(target: self, action: #selector(onDrag(gesture:)))
        addGestureRecognizer(Drag)
    }

    required init?(coder _: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }

    deinit {
        BFLog(message: "BFRecordAvatarView deinit")
//        closeCamera()
    }

    @objc func cameraClosed() {
        isHidden = true
//        closeCamera()
    }

    // 打开摄像头
    func openCamera() {
        camera.startCapture()
        let filter = SketchFilter()
        camera.addTarget(filter, atTargetIndex: 0)
        filter.addTarget(renderView, atTargetIndex: 0)
    }

    // 关闭摄像头
    func closeCamera() {
        camera.stopCapture()
    }

    // 开始录制
    func beginRecord(startTime: CMTime = .zero) {
        if !camera.captureSession.isRunning {
            BFLog(message: "摄像机不可用")
            return
        }
        startRecordeTime = startTime
        BFLog(1, message: "开始录制时间:\(CMTimeGetSeconds(startRecordeTime))")

        do {
            
            movieOutput!.startRecording()
            BFLog(1, message: "开始录制时间2:\(Date())")

        } catch {
            fatalError("Couldn't initialize movie, error: \(error)")
        }
    }

    // 结束录制
    func endRecord() {
        movieOutput?.finishRecording {
            self.camera.audioEncodingTarget = nil
            self.movieOutput = nil

            if let filePath = self.recorderFilePath, filePath.count > 0 {
                BFLog(message: "录制成功:\(self.recorderFilePath ?? "")")
                self.recorderFilePath = filePath.replacingOccurrences(of: documensDirectory, with: "")

                // 拼接回调 model
                let movieAsset = AVURLAsset(url: URL(fileURLWithPath: filePath), options: avAssertOptions)

                let movieInfo: PQEditVisionTrackMaterialsModel = PQEditVisionTrackMaterialsModel()
                movieInfo.type = StickerType.VIDEO.rawValue
                movieInfo.locationPath = self.recorderFilePath ?? ""
                movieInfo.timelineIn = Float64(CMTimeGetSeconds(self.startRecordeTime))
                movieInfo.timelineOut = CMTimeGetSeconds(movieAsset.duration)
                movieInfo.model_in = 0
                movieInfo.out = movieInfo.timelineOut
                movieInfo.canvasFillType = stickerContentMode.aspectFitStr.rawValue

                if self.recordEndCallBack != nil {
                    self.recordEndCallBack!(false, movieInfo)
                }

            } else {
                BFLog(message: "录制失败:\(self.recorderFilePath ?? "")")
                if self.recordEndCallBack != nil {
                    self.recordEndCallBack!(false, nil)
                }
            }
        }
    }

//
//                let movieAsset = AVURLAsset(url: URL(fileURLWithPath: fileURL), options: avAssertOptions)
//
//                let movieInfo: PQEditVisionTrackMaterialsModel = PQEditVisionTrackMaterialsModel()
//                movieInfo.type = StickerType.VIDEO.rawValue
//                movieInfo.locationPath = ""
//                movieInfo.timelineIn = 0
//                movieInfo.timelineOut = CMTimeGetSeconds(movieAsset.duration)
//                movieInfo.model_in = movieInfo.timelineIn
//                movieInfo.out = movieInfo.timelineOut
//                movieInfo.canvasFillType = stickerContentMode.aspectFitStr.rawValue
//                recordEndCallBack!(movieInfo)
//            }

    // 拖拽手势方法具体实现
    @objc func onDrag(gesture: UIPanGestureRecognizer) {
        let translation = gesture.translation(in: superview)
        var center = gesture.view!.center
        BFLog(message: "translationtranslationtranslation\(center.x) \(center.y)")
        center.x += translation.x
        center.y += translation.y
        if center.x > bounds.width / 2, center.x < cScreenWidth - bounds.width / 2, center.y > bounds.height / 2, center.y < cScreenHeigth - bounds.height / 2 {
            gesture.view!.center = center
            gesture.setTranslation(CGPoint.zero, in: superview)
        }
    }
}
