//
import BFCommonKit
import BFMediaKit
import BFUIKit
//  PQVoiceCell.swift
//  PQSpeed
//
//  Created by ak on 2020/8/18.
//  Copyright © 2020 BytesFlow. All rights reserved.
//
import Foundation
import UIKit

class PQVoiceCell: UICollectionViewCell {
 
    // 头像
    lazy var coverView: UIImageView = {
        let coverView = UIImageView()
        coverView.backgroundColor = UIColor.clear
        coverView.addCorner(corner: 25)

        return coverView
    }()

    lazy var nameLab: UILabel = {
        let nameLab = UILabel()
        nameLab.font = UIFont.systemFont(ofSize: 15)
        nameLab.textAlignment = .center
        return nameLab
    }()

    lazy var markView: UIImageView = {
        let markView = UIImageView()
        markView.isHidden = true
        return markView
    }()

    // 语音加载中动画
    lazy var audioLoding: UIImageView = {
        let audioLoding = UIImageView(frame: self.frame)
        audioLoding.image = imageInRecordScreenKit(by: "audioLoding")?.withRenderingMode(.alwaysTemplate)
        audioLoding.isHidden = true
        audioLoding.tintColor = .white
        return audioLoding
    }()

    // 语音播放动画和暂停播放
    lazy var audioPlaying: UIImageView = {
        let playImageView = UIImageView()
        playImageView.image = imageInRecordScreenKit(by: "stuckPoint_music_pause")
        playImageView.contentMode = .scaleAspectFit
        return playImageView
    }()

    override init(frame: CGRect) {
        super.init(frame: frame)
        contentView.addSubview(coverView)
        contentView.addSubview(markView)
        contentView.addSubview(nameLab)
        contentView.addSubview(audioLoding)
        contentView.addSubview(audioPlaying)
    }

    required init?(coder _: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }

    @objc class func voiceList(collectionView: UICollectionView, indexPath: IndexPath) -> PQVoiceCell {
        let cell: PQVoiceCell = collectionView.dequeueReusableCell(withReuseIdentifier: "PQVoiceCell", for: indexPath) as! PQVoiceCell
        return cell
    }

    var voice: PQVoiceModel? {
        didSet {
            addData()
            addLayout()
        }
    }

    func showLoading() {
        audioLoding.isHidden = false
    }

    func showPlay() {
        audioLoding.stopAnimating()
        audioLoding.isHidden = true
    }

    func addData() {
        nameLab.text = voice?.name

        coverView.setNetImage(url: "\(voice?.avatarUrl ?? "")")
        BFLog(message: "voice!.voiceStatue is \(voice?.name ?? "") : \(voice!.voiceStatue)")

        // 0控制蒙层
        if voice!.voiceStatue == .isNormal {
            markView.isHidden = true
        } else {
            markView.isHidden = false
            if voice!.voiceStatue == .isSelected {
                markView.image = imageInRecordScreenKit(by: "markSeleced")
            } else {
                markView.image = imageInRecordScreenKit(by: "markPlaying")
            }
        }

        // 1控制文字
        nameLab.textColor = voice!.voiceStatue == .isNormal ? UIColor.white : UIColor.hexColor(hexadecimal: "#389AFF")

        // 2控制播放状态
        if voice!.voiceStatue == .isPlaying {
            BFLog(message: "设置到播放状态！")
            audioPlaying.isHidden = false
            audioPlaying.kf.setImage(with: URL(fileURLWithPath: currentBundle()!.path(forResource: "stuckPoint_music_playing", ofType: ".gif")!))
        } else if voice!.voiceStatue == .isPause {
            audioPlaying.isHidden = false
            audioPlaying.image = imageInRecordScreenKit(by: "stuckPoint_music_pause")

        } else {
            audioPlaying.isHidden = true
        }

        // 3控制加载中状态
        if voice!.voiceStatue == .isLoading {
            audioLoding.layer.removeAllAnimations()
            audioLoding.isHidden = false
            // 1.创建动画
            let rotationAnim = CABasicAnimation(keyPath: "transform.rotation.z")
            // 2.设置动画的属性
            rotationAnim.fromValue = 0
            rotationAnim.toValue = Double.pi * 2
            rotationAnim.repeatCount = MAXFLOAT
            rotationAnim.duration = 1
            // 这个属性很重要 如果不设置当页面运行到后台再次进入该页面的时候 动画会停止
            rotationAnim.isRemovedOnCompletion = false
            // 3.将动画添加到layer中
            audioLoding.layer.add(rotationAnim, forKey: nil)
        } else {
            audioLoding.layer.removeAllAnimations()
            audioLoding.isHidden = true
        }
    }

    func addLayout() {
        coverView.snp.makeConstraints { make in
            make.top.equalToSuperview()
            make.centerX.equalToSuperview()
            make.width.height.equalTo(50)
        }

        nameLab.snp.makeConstraints { make in
            make.top.equalTo(coverView.snp.bottom).offset(7)
            make.centerX.equalToSuperview()
            make.bottom.equalToSuperview()
        }

        audioLoding.snp.makeConstraints { make in
            make.center.equalTo(coverView.snp.center)
            make.height.width.equalTo(18)
        }
        audioPlaying.snp.makeConstraints { make in
            make.center.equalTo(coverView.snp.center)
            make.height.width.equalTo(18)
        }
        markView.snp.makeConstraints { make in
            make.size.equalTo(coverView)
            make.centerX.equalTo(coverView)
        }
    }
}
