//
//  BFRecordItemModel.swift
//  BFRecordScreenKit
//
//  Created by 胡志强 on 2021/12/6.
//

import BFCommonKit
import BFMediaKit
import Foundation
import Photos

struct SplitRecordRange {
    var isRecord: Bool = false
    var range: CMTimeRange
    var index: Int
}

public enum ModelType : Int {
    case Image
    case Video
    case Camera
}

public class BFRecordItemModel: NSObject {
//    var baseMaterial : AVURLAsset?
    var localPath: String?
    var coverPath: String?
    var materialDuraion: CMTime = .zero
    var fetchCoverImgCallBack: ((UIImage) -> Void)?
    var fetchPlayItemCallBack: ((BFRecordItemModel?) -> Void)?
    var dealedDurationRanges = [SplitRecordRange]() // 录音切割的时间区间，合成导出时计算
    public var voiceStickers = [PQVoiceModel]() //
    //add by ak 保存有发音人转后的语音
    public var voiceChangeStickers = [PQVoiceModel]() //
    public var videoStickers = [PQEditVisionTrackMaterialsModel]() // 合成导出时计算
//    public var imageStickers = [PQEditVisionTrackMaterialsModel]() //
    public var titleStickers = [PQEditSubTitleModel]() // 字幕贴纸

    var events = [WithDrawModel]() // 行为记录，方便撤销

    public var coverImg: UIImage? // 封面图
    public var thumbImgs = [UIImage]() // 缩略图集合
    public var playItem: AVPlayerItem? // 视频playerItem
    public var videoAsset: AVURLAsset? // 视频Asset
    public var mediaType: ModelType? // 素材类型
    public var progress: Double = 0 // 更新进度
    public var index = 0 // 素材index
    public var width = 0 // 素材宽
    public var height = 0 // 素材高
    public var videoDegress: UInt = 0 // 视频拍摄角度 90，270为横屏，180，0为竖屏

    public override func mutableCopy() -> Any {
        let new = BFRecordItemModel()
        new.localPath  = self.localPath
        new.coverPath  = self.coverPath
        new.materialDuraion  = self.materialDuraion
        new.fetchCoverImgCallBack  = self.fetchCoverImgCallBack
        new.fetchPlayItemCallBack  = self.fetchPlayItemCallBack

        
        self.voiceStickers.forEach { mod in
            new.voiceStickers.append(mod.copy() as! PQVoiceModel)
        }
        self.voiceChangeStickers.forEach { mod in
            new.voiceChangeStickers.append(mod.copy() as! PQVoiceModel)
        }
        self.videoStickers.forEach { mod in
            let copymod = mod.copy() as! PQEditVisionTrackMaterialsModel
            copymod.thumImgs = mod.thumImgs
            new.videoStickers.append(copymod)
        }
        self.titleStickers.forEach { mod in
            new.titleStickers.append(mod.copy() as! PQEditSubTitleModel)
        }
        new.thumbImgs.append(contentsOf: self.thumbImgs)
        
        new.events = self.events
        new.coverImg    = self.coverImg
        new.playItem    = self.playItem
        new.videoAsset  = self.videoAsset
        new.mediaType   = self.mediaType
        new.progress    = self.progress
        new.index       = self.index
        new.width       = self.width
        new.height      = self.height
        new.videoDegress  = self.videoDegress

        return new
    }
    
    func initOriginData(phasset: PHAsset) {
        width = phasset.pixelWidth
        height = phasset.pixelHeight
        materialDuraion = CMTime.init(seconds: phasset.duration, preferredTimescale: 1000)
        fetchCoverImage(phasset)
//        fetchAVUrlAsset(phasset)

        if phasset.mediaType == .image {
            mediaType = .Image
            localPath = "image"
        } else if phasset.mediaType == .video {
            mediaType = .Video
            fetchPlayItem(phasset)
        }
    }

    func fetchCoverImage(_ phasset: PHAsset) {
        let option = PHImageRequestOptions()
        option.isNetworkAccessAllowed = true // 允许下载iCloud的图片
        option.resizeMode = .fast
        option.deliveryMode = .highQualityFormat
        PHImageManager.default().requestImage(for: phasset,
                                              targetSize: CGSize(width: width, height: height),
                                              contentMode: .aspectFit,
                                              options: option) { [weak self] image, _ in
            // 设置首帧/封面
            if image != nil {
                self?.coverImg = image
                if self?.mediaType == .Image {
                    var size = CGSize.zero
                    let rate = image!.size.width / image!.size.height
                    if image!.size.width > image!.size.height {
                        size.height = 50 * UIScreen.main.scale
                        size.width = size.height * rate
                    }else {
                        size.width = 70 * UIScreen.main.scale
                        size.height = size.width / rate
                    }
                    
                    if let thumImage = image?.nx_scaleToSize(size: size) {
                        self?.thumbImgs.append(thumImage)
                    }
                }
                self?.fetchCoverImgCallBack?(image!)
            }
        }
    }

    func fetchPlayItem(_ phasset: PHAsset) {
        PQPHAssetVideoParaseUtil.parasToAVPlayerItem(phAsset: phasset) { [weak self] playerItem, _, _ in
            
            guard let wself = self else { return }
            
            guard let item = playerItem else {
                self?.fetchPlayItemCallBack?(self)
                return
            }
            wself.playItem = item
            wself.videoAsset = item.asset as? AVURLAsset
            wself.localPath = (wself.videoAsset?.url.absoluteString.removingPercentEncoding)?.replacingOccurrences(of: "file://", with: "")

            wself.fetchPlayItemCallBack?(self)
        }
    }

    public func fetchAVUrlAsset(_ phasset: PHAsset) {
        PQPHAssetVideoParaseUtil.parasToAVAsset(phAsset: phasset) { [weak self] asset, _, _, _ in
            guard let wself = self else {
                return
            }
            if let videoAsset = (asset as? AVURLAsset) {
                wself.localPath = (videoAsset.url.absoluteString.removingPercentEncoding)?.replacingOccurrences(of: "file://", with: "")
                wself.videoAsset = videoAsset
//                wself.fetchAVUrlAssetCallBack?()
            }
        }
    }

    func generationTimeRanges(needSort _: Bool = false) {
        dealedDurationRanges.removeAll()

        var start: CMTime = .zero

        var list: [PQVoiceModel]
        list = voiceStickers.sorted { model1, model2 in
            model1.startCMTime.seconds < model2.endCMTime.seconds
        }

        for model in list {
            if CMTimeCompare(model.startCMTime, start) > 0{
                let range = CMTimeRange(start: start, duration: model.startCMTime - start)
                dealedDurationRanges.append(SplitRecordRange(isRecord: false, range: range, index: -1))
            }
            let ind = voiceStickers.firstIndex(of: model)
            let range = CMTimeRange(start: model.startCMTime, end: model.endCMTime)
            dealedDurationRanges.append(SplitRecordRange(isRecord: true, range: range, index: ind ?? -1))
            start = model.endCMTime
        }
        if CMTimeCompare(start, materialDuraion) < 0 {
            let range = CMTimeRange(start: start, end: materialDuraion)
            dealedDurationRanges.append(SplitRecordRange(isRecord: false, range: range, index: -1))
        }
    }

    /// 视频分解成帧
    /// - parameter frames                          : 需要取的帧数
    /// - parameter firstImagesCount             : 获取首先N张连续视频帧后先返回给调用方使用作为缓冲
    /// - parameter splitCompleteClosure    : 回调
    func splitVideoFileUrlFps(ass:AVURLAsset? = nil, frames: Int, firstImagesCount: Int = 0, maxSize : CGSize = CGSize(width: 200, height: 200), splitCompleteClosure: @escaping ((Bool, [UIImage]) -> Void)) {
        var urlAsset = ass
        if urlAsset == nil {
            urlAsset = videoAsset
        }
        if urlAsset == nil || urlAsset!.duration.seconds <= 0 {
            return
        }

        var splitImages = [UIImage]()

        var times = [NSValue]()

//        let urlAsset = AVURLAsset(url: URL(fileURLWithPath: localPath))
        let start = 0
//        let end = Int(urlAsset.duration.seconds * Float64(fps))
        let fps = Double(frames) / urlAsset!.duration.seconds
        for i in start ..< frames {
            let timeValue = NSValue(time: CMTimeMake(value: Int64(i * 1000), timescale: Int32(fps * 1000)))

            times.append(timeValue)
        }

        let imgGenerator = AVAssetImageGenerator(asset: urlAsset!)
        imgGenerator.requestedTimeToleranceBefore = CMTime.zero
        imgGenerator.requestedTimeToleranceAfter = CMTime.zero
        imgGenerator.appliesPreferredTrackTransform = true
        imgGenerator.maximumSize = maxSize
        let timesCount = times.count
        var cocu = 0
        // 获取每一帧的图片
        imgGenerator.generateCGImagesAsynchronously(forTimes: times) { _, image, _, result, _ in
            cocu += 1
            switch result {
            case AVAssetImageGenerator.Result.cancelled:
                BFLog(1, message: "splitVideo: cancel")

            case AVAssetImageGenerator.Result.failed:
                BFLog(1, message: "splitVideo: failed")

            case AVAssetImageGenerator.Result.succeeded:
                let framImg = UIImage(cgImage: image!)
                splitImages.append(framImg)
            //            BFLog(1, message: "aaa: \(requestedTime.seconds) - \(actualTime.seconds)")
            @unknown default:
                break
            }

            if cocu == firstImagesCount {
                splitCompleteClosure(false, splitImages)
            }

            if cocu == timesCount { // 最后一帧时 回调赋值
                splitCompleteClosure(true, splitImages)
                BFLog(1, message: "splitVideo: complete")
            }
        }
    }

    /// 视频分解成帧
    /// - parameter fileUrl                 : 视频地址
    /// - parameter fps                     : 自定义帧数 每秒内取的帧数
    /// - parameter splitCompleteClosure    : 回调
    func getThumbImageAtTime(urlAsset: AVURLAsset, time: CMTime) -> UIImage? {
        let imgGenerator = AVAssetImageGenerator(asset: urlAsset)
        imgGenerator.requestedTimeToleranceBefore = CMTime.zero
        imgGenerator.requestedTimeToleranceAfter = CMTime.zero

        var cgImg = try? imgGenerator.copyCGImage(at: time, actualTime: nil)
        if cgImg == nil {
            imgGenerator.requestedTimeToleranceBefore = CMTime.positiveInfinity
            imgGenerator.requestedTimeToleranceAfter = CMTime.positiveInfinity
            cgImg = try? imgGenerator.copyCGImage(at: time, actualTime: nil)
        }

        return cgImg == nil ? nil : UIImage(cgImage: cgImg!)
    }
    
    
    /// add by ak 返回真正要使用的音频数据，如果有变音使用变音数据
    /// - Returns: 最后使用的数据
    func getUsedVoices() ->[PQVoiceModel]{
        
        return voiceChangeStickers.count > 0 ? voiceChangeStickers : voiceStickers
    }
    
}
